/**
 * @file    mdf100a_it.c
 * @author  MegawinTech Application Team
 * @version V1.0.0
 * @date    01-October-2024
 * @brief   This file contains all the system functions
 */

/* Define to prevent recursive inclusion */
#define _MDF100A_IT_C_

/* Files include */
#include "platform.h"
#include "usart_dma_interrupt.h"
#include "mdf100a_it.h"

/**
  * @addtogroup MDF100A_LibSamples
  * @{
  */

/**
  * @addtogroup USART
  * @{
  */

/**
  * @addtogroup USART_DMA_Interrupt
  * @{
  */

/* Private typedef ****************************************************************************************************/

/* Private define *****************************************************************************************************/

/* Private macro ******************************************************************************************************/

/* Private variables **************************************************************************************************/

/* Private functions **************************************************************************************************/

/***********************************************************************************************************************
  * @brief  This function handles NMI exception
  * @note   none
  * @param  none
  * @retval none
  *********************************************************************************************************************/
void NMI_Handler(void)
{
}

/***********************************************************************************************************************
  * @brief  This function handles Hard Fault exception
  * @note   none
  * @param  none
  * @retval none
  *********************************************************************************************************************/
void HardFault_Handler(void)
{
    /* Go to infinite loop when Hard Fault exception occurs */
    while (1)
    {
    }
}

/***********************************************************************************************************************
  * @brief  This function handles SVCall exception
  * @note   none
  * @param  none
  * @retval none
  *********************************************************************************************************************/
void SVC_Handler(void)
{
}

/***********************************************************************************************************************
  * @brief  This function handles PendSVC exception
  * @note   none
  * @param  none
  * @retval none
  *********************************************************************************************************************/
void PendSV_Handler(void)
{
}

/***********************************************************************************************************************
  * @brief  This function handles SysTick Handler
  * @note   none
  * @param  none
  * @retval none
  *********************************************************************************************************************/
void SysTick_Handler(void)
{
    if (0 != PLATFORM_DelayTick)
    {
        PLATFORM_DelayTick--;
    }
}

/***********************************************************************************************************************
  * @brief  This function handles DMA1_Channel1 Handler
  * @note   none
  * @param  none
  * @retval none
  *********************************************************************************************************************/
void DMA1_Channel1_IRQHandler(void)
{
    if (RESET != DMA_GetITStatus(DMA1_IT_TC1))
    {
        DMA_Cmd(DMA1_Channel1, DISABLE);

        USART_TX_DMA_InterruptFlag = 1;

        DMA_ClearITPendingBit(DMA1_IT_TC1);
    }
}

/***********************************************************************************************************************
  * @brief  This function handles DMA1_Channel2 Handler
  * @note   none
  * @param  none
  * @retval none
  *********************************************************************************************************************/
void DMA1_Channel2_IRQHandler(void)
{
    if (RESET != DMA_GetITStatus(DMA1_IT_TC2))
    {
        DMA_Cmd(DMA1_Channel2, DISABLE);

        USART_RX_DMA_InterruptFlag = 1;

        DMA_ClearITPendingBit(DMA1_IT_TC2);
    }
}

/**
  * @}
  */

/**
  * @}
  */

/**
  * @}
  */


