/**
 * @file    tim2_pwm_inputcapture.c
 * @author  MegawinTech Application Team
 * @version V1.0.0
 * @date    01-October-2024
 * @brief   This file contains all the system functions 
 */

/* Define to prevent recursive inclusion */
#define _TIM2_PWM_INPUTCAPTURE_C_

/* Files include */
#include <stdio.h>
#include "platform.h"
#include "tim2_pwm_inputcapture.h"

/**
  * @addtogroup MDF101A_LibSamples
  * @{
  */

/**
  * @addtogroup TIM2
  * @{
  */

/**
  * @addtogroup TIM2_PWM_InputCapture
  * @{
  */

/* Private typedef ****************************************************************************************************/

/* Private define *****************************************************************************************************/

/* Private macro ******************************************************************************************************/

/* Private variables **************************************************************************************************/

/* Private functions **************************************************************************************************/

/***********************************************************************************************************************
  * @brief
  * @note   none
  * @param  none
  * @retval none
  *********************************************************************************************************************/
void TIM2_Configure(void)
{
    GPIO_InitTypeDef        GPIO_InitStruct;
    NVIC_InitTypeDef        NVIC_InitStruct;
    RCC_ClocksTypeDef       RCC_Clocks;
    TIM_TimeBaseInitTypeDef TIM_TimeBaseStruct;
    TIM_ICInitTypeDef       TIM_ICInitStruct;

    RCC_GetClocksFreq(&RCC_Clocks);

    RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM2, ENABLE);

    TIM_TimeBaseStructInit(&TIM_TimeBaseStruct);
    TIM_TimeBaseStruct.TIM_Prescaler         = (RCC_Clocks.PCLK1_Frequency / 1000000 - 1);
    TIM_TimeBaseStruct.TIM_CounterMode       = TIM_CounterMode_Up;
    TIM_TimeBaseStruct.TIM_Period            = 0xFFFFFFFF;
    TIM_TimeBaseStruct.TIM_ClockDivision     = TIM_CKD_Div1;
    TIM_TimeBaseStruct.TIM_RepetitionCounter = 0;
    TIM_TimeBaseInit(TIM2, &TIM_TimeBaseStruct);

    TIM_ICStructInit(&TIM_ICInitStruct);
    TIM_ICInitStruct.TIM_Channel     = TIM_Channel_1;
    TIM_ICInitStruct.TIM_ICPolarity  = TIM_ICPolarity_Falling;
    TIM_ICInitStruct.TIM_ICSelection = TIM_ICSelection_DirectTI;
    TIM_ICInitStruct.TIM_ICPrescaler = TIM_ICPSC_Div1;
    TIM_ICInitStruct.TIM_ICFilter    = 0x00;
    TIM_PWMIConfig(TIM2, &TIM_ICInitStruct);

    TIM_SelectSlaveMode(TIM2, TIM_SlaveMode_Reset);
    TIM_SelectInputTrigger(TIM2, TIM_TS_TI1FP1);

    TIM_ClearFlag(TIM2, TIM_FLAG_CC1);
    TIM_ITConfig(TIM2, TIM_IT_CC1, ENABLE);

    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA, ENABLE);

    GPIO_PinAFConfig(GPIOA, GPIO_PinSource6, GPIO_AF_0);    /* TIM2_CH1 */

    GPIO_StructInit(&GPIO_InitStruct);
    GPIO_InitStruct.GPIO_Pin   = GPIO_Pin_6;
    GPIO_InitStruct.GPIO_Speed = GPIO_Speed_High;
    GPIO_InitStruct.GPIO_Mode  = GPIO_Mode_FLOATING;
    GPIO_Init(GPIOA, &GPIO_InitStruct);

    NVIC_InitStruct.NVIC_IRQChannel = TIM2_IRQn;
    NVIC_InitStruct.NVIC_IRQChannelPriority = 0x01;
    NVIC_InitStruct.NVIC_IRQChannelCmd = ENABLE;
    NVIC_Init(&NVIC_InitStruct);

    TIM_Cmd(TIM2, ENABLE);
}

/***********************************************************************************************************************
  * @brief
  * @note   none
  * @param  none
  * @retval none
  *********************************************************************************************************************/
void TIM2_PWM_InputCapture_Sample(void)
{
    float Frequency, Duty;

    printf("\r\nTest %s", __FUNCTION__);

    printf("\r\nInjection frequency range : [20Hz, 100kHz]");

    TIM2_Configure();

    while (1)
    {
        if (1 == TIM2_CC_InterruptFlag)
        {
            Frequency = (float)1000000 / (float)TIM2_CC_Value1;

            Duty = (float)TIM2_CC_Value2 / (float)TIM2_CC_Value1 * (float)100.0;

            if (Frequency >= 1000000.0)
            {
                printf("\r\nFrequency = %0.1fMHz, Duty = %0.1f%%", Frequency / (float)1000000.0, Duty);
            }
            else if (Frequency >= 1000.0)
            {
                printf("\r\nFrequency = %0.1fkHz, Duty = %0.1f%%", Frequency / (float)1000.0, Duty);
            }
            else
            {
                printf("\r\nFrequency = %0.1fHz,  Duty = %0.1f%%", Frequency, Duty);
            }

            TIM2_CC_InterruptFlag = 0;
        }
    }
}

/**
  * @}
  */

/**
  * @}
  */

/**
  * @}
  */



